// Imports
const fs = require("fs");
const { resolve: rsPath } = require("path");
const multer = require("multer");
// Constants
const bucketsPath = process.env.NUBIAN_BUCKET_PATH ?? "buckets/";
// Methods
const bucketStorage = (bucketStoragePath, containerCb) =>
  multer.diskStorage({
    destination: (req, file, cb) =>
      cb(null, upsertDestination(req, bucketStoragePath, containerCb)),
    filename: (req, file, cb) => {
      const n = file.originalname.replaceAll(" ", "_");
      const fileName = `${Date.now()}-${n}`;
      req.on("aborted", () => cancelUpload(req, fileName));
      cb(null, fileName);
    },
  });

const buildBucketUpload = (bucketStoragePath, fieldName, containerCb) =>
  multer({
    storage: bucketStorage(bucketStoragePath, containerCb),
  }).single(fieldName);

const cancelUpload = (req, fileName) => {
  const path = rsPath(req.bucketStoragePath, fileName);
  deletePath(path);
};

const deletePath = (path) => fs.existsSync(path) && fs.unlinkSync(path);

const deleteUpload = (bucketStoragePath, container, filename) =>
  deletePath(fetchUpload(bucketStoragePath, container, filename));

const fetchUpload = (bucketStoragePath, container, filename) =>
  rsPath(bucketsPath, bucketStoragePath, container, filename);

const upsertDestination = (req, bucketStoragePath, containerCb) => {
  var container = "";
  if (containerCb) {
    if (typeof containerCb !== "function")
      throw Error("Container Callback must be a function!");
    container = containerCb(req);
  }
  req.bucketStoragePath = rsPath(bucketsPath, bucketStoragePath, container);
  fs.mkdirSync(req.bucketStoragePath, { recursive: true });
  return req.bucketStoragePath;
};

module.exports = {
  buildBucketUpload,
  cancelUpload,
  deleteUpload,
  fetchUpload,
};
